\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Centroid of boundary segments.


read("../tools/memoize.gp")

check_equal(got,want) =
{
  if(got!=want, print("wrong\n","got\n",got,"\nwant\n",want);
                quit(1),
                print("ok"));
}
vector_mean(v) = vecsum(v)/#v;
R(k) = if(k==0, 1, 3*2^(k-1));
V(k) = if(k==0, 2, 3*2^(k-1));

\\ (1 + sqrt(3)*I)/2
\\ D=-3
sqrt3i = quadgen(4*-3);
w6 = 1/2 + 1/2*sqrt3i;
check_equal(w6^3, -1)
check_equal(w6^6, 1)
w3 = -1/2 + 1/2*sqrt3i;
check_equal(w3^3, 1)

w3 = w6^2;
b = w6 + 1;

terdragon_n_to_xy(n) = {
  my(v=digits(n,3),
     e=1, dir=1,
     ret=0);
  for(i=1,#v,  \\ high to low
     ret *= b;
     if(v[i]==1, ret+=1*dir; dir*=w3,
        v[i]==2, ret+=w6*dir));
  ret;
}
check_equal(terdragon_n_to_xy(0), 0)
check_equal(terdragon_n_to_xy(1), 1)
check_equal(terdragon_n_to_xy(2), w6)
check_equal(terdragon_n_to_xy(3), b)

check_equal(terdragon_n_to_xy(4), b+w3)
check_equal(terdragon_n_to_xy(5), b-1)
check_equal(terdragon_n_to_xy(6), b+w3*b)

check_equal(terdragon_n_to_xy(7), b^2-w6)
check_equal(terdragon_n_to_xy(8), b^2-1)
check_equal(terdragon_n_to_xy(9), b^2)

terdragon_midpoints(k) = {
  my(e=1,v=[1/2]);
  for(i=1,k,
     my(p2 = e*w6);
     v = concat(v,
                concat(apply(z->e+w3*z, v),
                       apply(z->p2+z,   v)));
     e *= b);
  v;
}
terdragon_end(k) = b^k;
terdragon_centroid(k) = terdragon_end(k)/2;

m0 = 1/2;
m1 = b/2;
m2 = b-1/2;
check_equal(terdragon_midpoints(0), [m0])
check_equal(terdragon_midpoints(1), [m0, m1, m2])

GR_samples = {[
(2/2 + 0*sqrt3i/2) * 1/2/1,
(9/2 + 3*sqrt3i/2) * 1/2/3,
(21/2 + 17*sqrt3i/2) * 1/2/6,
(24/2 + 70*sqrt3i/2) * 1/2/12,
(-117/2 + 233*sqrt3i/2) * 1/2/24,
(-1023/2 + 589*sqrt3i/2) * 1/2/48,
(-4812/2 + 770*sqrt3i/2) * 1/2/96,
(-16749/2 + -2447*sqrt3i/2) * 1/2/192,
(-42987/2 + -24007*sqrt3i/2) * 1/2/384,
(-57180/2 + -114926*sqrt3i/2) * 1/2/768,
(172755/2 + -401959*sqrt3i/2) * 1/2/1536
]};

GV_samples = {[
(-1/2 + 3*sqrt3i/2) * 1/2/2,
(-6/2 + 8*sqrt3i/2) * 1/2/3,
(-33/2 + 17*sqrt3i/2) * 1/2/6,
(-141/2 + 19*sqrt3i/2) * 1/2/12,
(-474/2 + -76*sqrt3i/2) * 1/2/24,
(-1197/2 + -683*sqrt3i/2) * 1/2/48,
(-1569/2 + -3217*sqrt3i/2) * 1/2/96,
(4866/2 + -11192*sqrt3i/2) * 1/2/192,
(48015/2 + -28711*sqrt3i/2) * 1/2/384,
(229935/2 + -38201*sqrt3i/2) * 1/2/768,
(804162/2 + 115088*sqrt3i/2) * 1/2/1536
]};

GR0 = 1/2;
GR1 = b/2;
check_equal(GR0, vector_mean(terdragon_midpoints(0)))
check_equal(GR1, vector_mean(terdragon_midpoints(1)))
check_equal(GR0, GR_samples[1])
check_equal(GR1, GR_samples[2])

GV0 = 1/2*(w3/2 + w3+1/2);
GV1 = 1/3*(w3/2 + 3/2*w3 + 2*w3+1/2);
check_equal(GV0, vector_mean(apply(z->z-1, terdragon_midpoints(1)[^1])))
check_equal(GV1, vector_mean([w3/2, 3/2*w3, 2*w3+1/2]))
check_equal(GV0, GV_samples[1])
check_equal(GV1, GV_samples[2])

GR(k) = {
  if(k==0, return(GR0));
  my(r=R(k-1),v=V(k-1)); \
  (r*GR(k-1)  + v*(b^(k-1) + GV(k-1)) )/(r+v);
}
GV(k) = {
  if(k==0, return(GV0));
  my(r=R(k-1),v=V(k-1));
  (r*w3*GR(k-1)  + v*(w3*b^(k-1) + GV(k-1)))/(r+v);
}
GR=memoize(GR);
GV=memoize(GV);


check_equal(GR(0), vector_mean(terdragon_midpoints(0)))
check_equal(vector_mean(terdragon_midpoints(1)), b/2)
check_equal(R(1), 3)
check_equal(R(0), 1)
check_equal(V(0), 2)
check_equal(GR(1), b/2)
check_equal(GR(1), vector_mean(terdragon_midpoints(1)))
check_equal(m0+m1+m2, 3*GR(1))

vector(8,k,my(k=k-1); real(GR_samples[k+1]))
vector(8,k,my(k=k-1); real(GR(k)))
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))
check_equal(GV_samples, \
            vector(#GV_samples,k,my(k=k-1); GV(k)))
GR(1)
GV(1)

\\                    *-----*
\\                     \
\\                      \
\\      *-----*    *---- *
\\       \          \   / \
\\        \          \ /   \
\\         O          *     O
\\
\\                    *------*
\\                     \
\\                      \
\\ O-------*      O------+
\\
GR(k) = {
  if(k==0, return(GR0));
  if(k==1, return(GR1));
  my(r=R(k-1),v=V(k-1)); \
  1/2*GR(k-1)  + 1/2*(b^(k-1) + GV(k-1));
}
GV(k) = {
  if(k==0, return(GV0));
  if(k==1, return(GV1));
  my(r=R(k-1),v=V(k-1));
  1/2*w3*GR(k-1)  + 1/2*(w3*b^(k-1) + GV(k-1));
}
GR=memoize(GR);
GV=memoize(GV);
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))
check_equal(GV_samples, \
            vector(#GV_samples,k,my(k=k-1); GV(k)))


GR(k) = {
  if(k==0, return(GR0));
  if(k==1, return(GR1));
  my(r=R(k-1),v=V(k-1)); \
  1/2*GR(k-1)  + 1/2*GV(k-1) + 1/2*b^(k-1);
}
GV(k) = {
  if(k==0, return(GV0));
  if(k==1, return(GV1));
  my(r=R(k-1),v=V(k-1));
  1/2*w3*GR(k-1) + 1/2*GV(k-1)  + 1/2*w3*b^(k-1);
}
GR=memoize(GR);
GV=memoize(GV);
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))
check_equal(GV_samples, \
            vector(#GV_samples,k,my(k=k-1); GV(k)))

\\ GR(k) = 1/2*GR(k-1) + 1/2*GV(k-1) + 1/2*b^(k-1);
check_equal(vector(20,k,my(k=k+1); GR(k)), \
            vector(20,k,my(k=k+1); 1/2*GR(k-1) + 1/2*GV(k-1) + 1/2*b^(k-1)))
check_equal(vector(20,k,my(k=k+1); GV(k)), \
            vector(20,k,my(k=k+1); 1/2*w3*GR(k-1) + 1/2*GV(k-1)  + 1/2*w3*b^(k-1)))

check_equal(vector(20,k,my(k=k+1); GR(k+1)), \
            vector(20,k,my(k=k+1); GR(k) - conj(b)/4*GR(k-1) + 1/4*b^(k+1)))
check_equal(vector(20,k,my(k=k+2); GR(k)), \
            vector(20,k,my(k=k+2); GR(k-1) - conj(b)/4*GR(k-2) + 1/4*b^k))

\\ subst into GV
check_equal(vector(20,k,my(k=k+2); GR(k)), \
            vector(20,k,my(k=k+2); GR(k-1) - conj(b)/4*GR(k-2) + 1/4*b^k))

GR(k) = {
  if(k==0, return(GR0));
  if(k==1, return(GR1));
  if(k==2, return(GR_samples[3]));
  GR(k-1) - conj(b)/4*GR(k-2) + 1/4*b^k;
}
GR=memoize(GR);
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))

polroots(x^2 - 1*x + conj(b)/4.0)
check_equal(x^2 - 1*x + conj(b)/4, \
            (x - (1/4 - sqrt3i/4))*(x - (3/4 + sqrt3i/4)))
check_equal(x^2 - 1*x + conj(b)/4, \
            (x + w3/2)*(x - b/2))

\\ g(x) = - x*(g(x)-GR0-GR1*x) + conj(b)/4*x^2*(g(x) - GR0) + 1/4*1/(1-b*x);
g(x) = {
  1/(1 + x - conj(b)/4*x^2)
  * ( -           x   * ( -GR0 - GR1*x - GR(2)*x^2 )
      + conj(b)/4*x^2 * ( -GR0 - GR1*x )
      + 1/4 * (1/(1 - b*x)  - 1 - b*x - b^2*x^2 - b^3*x^3 )
  )
  + GR0 + GR1*x + GR(2)*x^2 + GR(3)*x^3
};
valuation(g(x),'x)
\\ Vec(1/(1-b*x)-O(x^7))
Vec(g(x)-O(x^7))
GR_samples[1..7]

\\ GR = X*(-w3/2)^k + Y*(b/2)^k + Z*b^k
{for(i=2,7,
    print(matsolve([(-w3/2)^i, (b/2)^i, b^i;
                    (-w3/2)^(i+1), (b/2)^(i+1), b^(i+1);
                    (-w3/2)^(i+2), (b/2)^(i+2), b^(i+2)],
                   [GR_samples[i+1]; GR_samples[i+2]; GR_samples[i+3]])))
}

GR(k) = {
  if(k==0, return(GR0));
  if(k==1, return(GR1));
  if(k==2, return(GR_samples[3]));
  my(w=sqrt3i);
  (6/13 - 1/13*w)*b^k
  + (1/6 + 1/6*w)*(b/2)^k
  + (1/26 - 7/78*w)*(-w3/2)^k;
}
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))
check_equal((6/13 - 1/13*sqrt3i),   (7-2*w6)/13)
check_equal(( 1/6 + 1/6*sqrt3i),    w6/3)
check_equal((1/26 - 7/78*sqrt3i),   (5-7*w6)/39)
check_equal(conj(w6)/2, -w3/2)

GR(k) = {
  if(k==0, return(GR0));
  (7-2*w6)/13   * b^k
  + (5-7*w6)/39 * (conj(w6)/2)^k
  + w6/3        * (b/2)^k            \\ extra over GRT(k)
  ;
}
check_equal(GR_samples, \
            vector(#GR_samples,k,my(k=k-1); GR(k)))

GV(10)/b^10*1.0*13/sqrt(3)

(7-2*w6)/13*w3
