#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $max_level = 20;
printwrap("% generated by tools/$FindBin::Script\n");

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;
print "% GP-DEFINE  $var = {[\n";

sub xy_to_expression {
  my ($x,$y) = @_;

  # w3 == sqrt3i/2 - 1/2
  # x/2 + y*sqrt3i/2 == (x+y)/2 + y*w3
  #
  ($x+$y) % 2 == 0 or die;
  $x = ($x+$y)/2;

  if ($x == 0 && $y == 0) {
    return '0';
  }

  my $str = '';
  if ($x) {
    $str = $x;
  }
  if ($y) {
    if ($y > 0 && length($str)) { $str .= '+'; }
    $str .= ($y==1 ? "w3" : "$y*w3");
  }
  return $str;
}

my $ex = 2;
my $ey = 0;
my $points = [
              [0,0],
              [$ex,$ey],
             ];

foreach my $k (0 .. $max_level) {
  {
    my ($ax,$ay) = AltEnd($k);
    ($ex == $ax && $ey == $ay) or die;
  }

  print "% GP-DEFINE    /* $k */ [ ";
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    print $join, xy_to_expression($x,$y);
    $join = ',';
  }
  print " ]", ($k<$max_level ? "," : ""), "\n";

  ($points,$ex,$ey) = MyTerdragon::alternate_hull_expand($points,$ex,$ey);
}
print "% GP-DEFINE  ]};\n";

print "% end generated\n";
exit 0;
