#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_to_node_name', '@dir3_to_dx','@dir3_to_dy',
  '$alt_hullstouch_segs','$alt_hullstouch_odd_segs';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $conf = $alt_hullstouch_segs;
foreach my $pos (0,1) {
  my $prefix = ($pos ? 'o' : '');
  print "  \\begin{scope}[shift=(pos-$pos)]\n";
  {
    my $str = $alt_hullstouch_segs->print_to_str;
    $str =~ s/^/    % /mg;
    $str =~ s/ +$//mg;
    print $str;
  }

  my %seen_point;
  foreach my $seg (sort $alt_hullstouch_segs->segs_list) {
    my ($x,$y,$dir) = split /,/, $seg;
    foreach (0, 1) {
      my $name = $prefix . xy_to_node_name($x,$y);
      unless ($seen_point{$name}++) {
        my $blank = ($x==0 && $y==0) || ($x==2 && $y==0) ? '' : ' blank';
        print "    \\node ($name) at ($x/2,{$y*sqrt(3)/2}) [my$blank dot];\n";
      }
      $x += $dir3_to_dx[$dir];
      $y += $dir3_to_dy[$dir];
    }
  }

  $Printwrap = 4;
  foreach my $seg (sort $alt_hullstouch_segs->segs_list) {
    next if $seg eq '0,0,0';
    my ($x,$y,$dir) = split /,/, $seg;
    my $from = $prefix . xy_to_node_name($x,$y);
    my $to   = $prefix . xy_to_node_name($x+$dir3_to_dx[$dir],
                                         $y+$dir3_to_dy[$dir]);
    printwrap(" \\draw[my arrow] ($from)--($to);");
  }
  printwrap_newline();
print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
