#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'Lpred_k','dxdy_to_dir6','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);
}

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (end) at ($x/2,{$y*sqrt(3)/2});\n";
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n), $path->n_to_xy($n+1));
  }
  $Printwrap = 2;
  printwrap("\\draw[my line,xscale=1/2,yscale=sqrt(3)/2] ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap(" % $count points\n");
}

foreach my $n ($n_lo .. $n_hi-1) {
  if (Lpred_k($n,$level)) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    ($x,$y) = transform($x,$y);
    ($dx,$dy) = transform($dx,$dy);
    my $dir6 = dxdy_to_dir6($dx,$dy);
    my $angle = $dir6 * 60;
    print "  \\pic at ($x/2,{$y*sqrt(3)/2}) [transform shape,rotate=$angle]{my hull limit pic};\n";
  }
}

print "  % end generated\n";
exit 0;
