#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $level = 10;

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $denominator = 2 * 3**($level/2);

sub transform {
  my ($x,$y) = @_;
  foreach (1 .. $level/2) {
    ($x,$y) = xy_rotate_minus60($x,$y);
  }
  return ($x,$y);
}

my $R = 0;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
  $R++;
}

$Printwrap = 2;
printwrap("\\begin{scope}[scale=1/$denominator,yscale=sqrt(3)]\n");
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap (transform => \&transform);
printwrap(";");

$Printwrap = 2;
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");

printwrap("\\end{scope}\n");
print "% GP-Test  R($level) == $R\n";
print "  % end generated\n";
exit 0;
