#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'max';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'transform_to_horizontal';
use MyPrintwrap ':all';
use MyLineSegments;


my $level = 2;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\begin{scope}[xscale=1/2,yscale={sqrt(3)/2}]\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $n_start = 3*$n_hi;
my $n_end   = 5*$n_hi;
my ($ox,$oy) = $path->n_to_xy($n_end);
my $x_max = 0;

my $segments = MyLineSegments->new;
foreach my $n ($n_start .. $n_end-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        $x -= $ox;
                        $y -= $oy;
                        ($x,$y) = transform_to_horizontal($x,$y, $level);
                        return ($x,$y);
                      });
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");
print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
