#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::Geometry::Planar;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_rotate_plus60','hull_and_end';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
$level % 2 == 0 or die;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub stringize {
  my ($x,$y) = @_;
  if ($y) {
    $y = "{$y*sqrt(3)}";
  }
  return "($x,$y)";
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }
  $Printwrap = 2;
  printwrap("\\draw[my grey] ");
  $Printwrap = 4;
  $segments->printwrap(stringize => \&stringize);
  printwrap(";\n");
}

{
  my ($points,$ex,$ey) = hull_and_end($level);
  $ex == $x_hi or die;
  $ey == $y_hi or die;

  my $tests = '';
  foreach my $i (4,7,8, 4+7,14,1) {
    my $p = $points->[$i-1];
    my ($x,$y) = @$p;
    my $name = ($i <= 7 ? "P$i"
                : "P".($i-7)."dash");
    $tests .= "% GP-Test  $name($level) == $x/2 + $y*sqrt3i/2\n";
    my $xy = stringize($x,$y);
    print "  \\node ($name) at $xy [my dot];\n";
  }
  print $tests;

  my $planar = Math::Geometry::Planar->new;
  $planar->points([ map {[$_->[0], $_->[1]*sqrt(3)]} @$points ]);
  $planar = $planar->minrectangle;
  my $minpoints = $planar->points;
  $Printwrap = 2;
  printwrap("  \\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $p (@$minpoints) {
    my ($x,$y) = @$p;
    printwrap($join . sprintf('(%.4f,%.4f)', $x,$y));
    $join = '--';
  }
  printwrap("${join}cycle;\n");
}

{
  print "  \\coordinate (start) at (0,0);\n";
}
{
  my $xy = stringize($x_hi,$y_hi);
  print "  \\coordinate (end) at $xy;\n";
}

print "  % end generated\n";
exit 0;
