#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_rotate_plus60','mul','div','chamfer','xy_to_node_name';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $n_start = 2*$n_hi;
my $n_end   = 5*$n_hi;
my ($ox,$oy) = $path->n_to_xy($n_start);

print "  \\begin{scope}[xscale=1/2,yscale=sqrt(3)/2]\n";
$Printwrap = 2;
printwrap("  \\draw[my curve]");
$Printwrap += 2;
my $join = '';
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  $x -= $ox;
  $y -= $oy;
  my $sx = $x . chamfer($dx);
  my $sy = $y . chamfer($dy);
  $x += $dx;
  $y += $dy;
  my $tx = $x . chamfer(-$dx);
  my $ty = $y . chamfer(-$dy);
  printwrap("$join($sx,$sy)");
  $join = '--';
  printwrap("--($tx,$ty)");
}
printwrap("--cycle;\n");
print "  \\end{scope}\n";

$Printwrap = 2;
my %seen_node;
my @edges;
my $prev_name;
foreach my $n ($n_start .. $n_end-1, $n_start) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  $x -= $ox;
  $y -= $oy;
  ($x,$y) = mul($x,$y);
  ($dx,$dy) = xy_rotate_plus60($dx,$dy);
  $x += $dx;
  $y += $dy;
  ($x,$y) = div(3*$x,3*$y);
  my $name = xy_to_node_name($x,$y);
  unless ($seen_node{$name}++) {
    printwrap(" \\node ($name) at ($x/6,{$y/sqrt(3)/2}) [my dot];");
  }
  if (defined $prev_name && $name ne $prev_name) {
    push @edges, " ($name)--($prev_name)";
  }
  $prev_name = $name;
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw[my edge]");
$Printwrap += 2;
foreach my $str (@edges) {
  printwrap($str);
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
