#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'turn','@dir3_to_dx','@dir3_to_dy','chamfer';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\begin{scope}[xscale=1/2,yscale=sqrt(3)/2]\n";
my %seen;
my @nodes;
my $prev_txy;
{
  my ($x,$y) = $path->n_to_xy(5);
  my ($dx,$dy) = $path->n_to_dxdy(5);
  my $tx = $x . chamfer($dx);
  my $ty = $y . chamfer($dy);
  $prev_txy = "$tx,$ty";
}
my ($x,$y) = $path->n_to_xy(6);
my $prev_dir = 1;
my $prev_turn = -1;
my $join = '';
foreach my $side (0, 1, 2) {
  foreach my $n ($n_lo+6 .. $n_hi-1+6) {
    my $turn = ($n==0 ? 1 : turn($n));
    my $dir = ($prev_dir + $turn) % 3;

    my $prev_dx = $dir3_to_dx[$prev_dir];
    my $prev_dy = $dir3_to_dy[$prev_dir];

    my $dx = $dir3_to_dx[$dir];
    my $dy = $dir3_to_dy[$dir];

    my $sx = $x . chamfer(-$prev_dx);
    my $sy = $y . chamfer(-$prev_dy);
    my $tx = $x . chamfer($dx);
    my $ty = $y . chamfer($dy);
    # $sx = "{($sx)/2}";
    # $sy = "{($sy)*sqrt(3)/2}";
    # $tx = "{($tx)/2}";
    # $ty = "{($ty)*sqrt(3)/2}";

    if ($turn==-1 && ! $seen{"$x,$y"}++) {
      push @nodes, "$x/2,{$y*sqrt(3)/2}";
    }

    if ($turn==-1 && $prev_turn==-1) {
      if ($join) {
        printwrap(";");
      }
      $Printwrap = 2;
      printwrap(" \\draw[my R line] ($prev_txy)--($sx,$sy);");
      $Printwrap += 2;
      $join = '';
    }
    if (! $join) {
      $Printwrap = 2;
      printwrap(" \\draw[my L line]");
      $Printwrap += 2;
    }
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");
    $join = '--';

    $prev_turn = $turn;
    $prev_txy = "$tx,$ty";
    $prev_dir = $dir;
    $x += $dx;
    $y += $dy;
  }
}
if ($join) {
  printwrap(";\n");
}
print "  \\end{scope}\n";

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $node (@nodes) { printwrap("($node),"); }
printwrap("}");
printwrap(" { \\p node[my dot] };\n");

my $TTV = scalar(@nodes);
print "% GP-Test-Last  TTV($level) == $TTV\n";
print "  % end generated\n";
exit 0;
